import re
from pathlib import Path

from marko import Markdown, block
from marko.md_renderer import MarkdownRenderer

from dumas.lib.interpreters import interpreters



FENCED_CODE_LANG_REGEX = re.compile(r"dumas\[(?P<interpreter_name>[\d\w_]+)(@(?P<interpreter_id>[\w\d_-]+))?\]")


class Renderer(MarkdownRenderer):
    def __init__(self):
        super().__init__()

    def render_fenced_code(self, element: block.FencedCode) -> str:
        match = FENCED_CODE_LANG_REGEX.match(element.lang)

        if not match:
            return super().render_fenced_code(element)

        match_dict = match.groupdict()
        interpreter_id = match_dict.get("interpreter_id", "default")
        interpreter_name = match_dict["interpreter_name"]

        interpreter = interpreters[interpreter_name](interpreter_id)

        return self.render(interpreter.run(self.render_raw_text(element.children[0]), **{}))


def render_text(text, *, renderer=Renderer) -> str:
    return Markdown(renderer=renderer)(text)


def render_file(path_to_file: Path, *, renderer=Renderer) -> str:
    return render_text(path_to_file.read_text(), renderer=renderer)



# SPDX-FileCopyrightText: 2023-present Alvaro Leiva Geisse <aleivag@gmail.com>
#
# SPDX-License-Identifier: MIT
import sys

if __name__ == "__main__":
    from dumas.cli import dumas

    sys.exit(dumas())



import re
import textwrap
from functools import lru_cache
from typing import Dict, Type

import libcst as cst
from IPython.core.interactiveshell import InteractiveShell
from marko.block import FencedCode

from black import format_str, Mode as black_Mode



class Interpreter:
    def __init__(self, interpreter_id: str) -> None:
        self.interpreter_id = interpreter_id


@lru_cache
class Python(Interpreter):
    language = "python"

    def __init__(self, interpreter_id: str) -> None:
        super().__init__(interpreter_id)
        self.shell = InteractiveShell()


    def run(self, code, black: bool = True):
        execution_count = self.shell.execution_count

        out = self.shell.run_cell(code)
        out.raise_error()
        self.shell.execution_count += 1

        in_ps1 = f"In [{execution_count}]: "
        len_in_ps1 = len(in_ps1)
        indent_ps1 = " " * (len_in_ps1 - (1 + 3 + 1)) + "...: "

        out_ps1 = f"Out[{execution_count}]: "

        if out.success:
            if out.result:
                result = f"\n\n{out_ps1}{out.result}"
            else:
                result = ""
        else:
            result = f"{out_ps1}\n{out.error_in_exec.__repr__()}"

        module = cst.parse_module(code)

        header = module.with_changes(body=[]).code
        body = module.with_changes(header=[]).code.strip()
        if black:
            body = format_str(body, mode=black_Mode())

        render_body = in_ps1 + textwrap.indent(body, prefix=indent_ps1, predicate=lambda n: True)[len_in_ps1:]

        pattern = r"^```"
        # Replacement string
        replacement = r"\\```"
        # Use re.sub() to replace the matches
        content = re.sub(pattern, replacement, f"{header}{render_body}{result}", flags=re.MULTILINE)

        return FencedCode(("python", "", content))


interpreters: Dict[str, Type[Interpreter]] = {
    "python": Python,
}



# SPDX-FileCopyrightText: 2023-present Alvaro Leiva Geisse <aleivag@gmail.com>
#
# SPDX-License-Identifier: MIT
import sys
from pathlib import Path
from typing import Optional

import click

from dumas.__about__ import __version__
from dumas.lib import renderer


@click.group(context_settings={"help_option_names": ["-h", "--help"]}, invoke_without_command=True)
@click.version_option(version=__version__, prog_name="dumas")
def dumas():
    ...


@dumas.command()
@click.option("--output-file", metavar="OUTPUT_FILE", type=click.File(mode="w"), help="Output file", default="-")
@click.argument("input_file", type=Path)
def render_file(input_file: Path, output_file):
    """
    writes [INPUT FILE] into --out file
    """

    output_file.write(renderer.render_file(input_file))


@dumas.command()
@click.option(
    "--check/--execute",
    "check",
    default=False,
    help="execute (a.k.a do the rendering) or check (fail if there is need for rendering)",
)
@click.option("--output-dir", metavar="OUTPUT_DIR", type=Path, required=True, help="Output file")
@click.argument("input_dir", type=Path)
def render_dir(input_dir: Path, output_dir: Path, check: bool):
    input_dir = input_dir.resolve()
    output_dir = output_dir.resolve()

    for f in input_dir.glob("**/*.md"):
        output_text = renderer.render_file(f)
        finalf = output_dir / f.relative_to(input_dir)
        if check:
            if finalf.read_text() != output_text:
                click.echo(f"file {f} needs to be renderer", err=True)
                sys.exit(-1)
        else:
            finalf.write_text(output_text)

    if check:
        click.echo("No changes needed", err=True)
